@@ -38,6 +38,7 @@ Example:
 
 import json
 import sys
+from warnings import warn
 
 import duckdb
 import pandas
@@ -50,6 +51,7 @@ from ._jsonx import _transform_json
 from ._select import _select
 from ._sqlx import _escape_sql
 from ._sqlx import _autocommit
+from ._sqlx import _sqlid
 
 class LDLite:
 
@@ -68,6 +70,7 @@ class LDLite:
         self._quiet = False
         self.dbtype = 0
         self.db = None
+        self.login_token = None
 
     def _set_page_size(self, page_size):
         self.page_size = page_size
@@ -116,7 +119,15 @@ class LDLite:
         data = { 'username': self.okapi_user,
                 'password': self.okapi_password }
         resp = requests.post(self.okapi_url+'/authn/login', headers=hdr, data=json.dumps(data))
-        return resp.headers['x-okapi-token']
+        self.login_token = resp.headers['x-okapi-token']
+
+    def _check_okapi(self):
+        if self.login_token is None:
+            raise RuntimeError('connection to okapi not configured: use connect_okapi()')
+
+    def _check_db(self):
+        if self.db is None:
+            raise RuntimeError('database connection not configured: use connect_db() or connect_db_postgresql()')
 
     def connect_okapi(self, url, tenant, user, password):
         """Connects to an Okapi instance.
@@ -136,37 +147,59 @@ class LDLite:
         self.okapi_tenant = tenant
         self.okapi_user = user
         self.okapi_password = password
-        # Test connection
-        _ = self._login()
+        self._login()
 
-    def query(self, table, path, query, transform=True):
+    def query(self, table, path, query, json_depth=3, transform=None):
         """Submits a CQL query to an Okapi module, and transforms and stores the result.
 
         The *path* parameter is the request path, and *query* is the CQL query.
-        The result is stored in *table* within the analytic database.  If
-        *transform* is True (the default), JSON data are transformed into one
-        or more tables that are created in addition to *table*.  New tables add
-        a suffix "_j" to *table* and overwrite any existing tables with the
-        same name.  A list of newly created tables is returned by this
-        function.
+        The result is stored in *table* within the analytic database.
+
+        By default JSON data are transformed into one or more tables that are
+        created in addition to *table*.  New tables overwrite any existing
+        tables having the same name.  If *json_depth* is specified within the
+        range 0 < *json_depth* < 5, this determines how far into nested JSON
+        data the transformation will descend.  (The default is 3.)  If
+        *json_depth* is specified as 0, JSON data are not transformed.
+
+        The *transform* parameter is no longer supported and will be removed in
+        the future.  Instead, specify *json_depth* as 0 to disable JSON
+        transformation.
+
+        A list of newly created tables is returned by this function.
 
         Example:
 
             ld.query(table='g', path='/groups', query='cql.allRecords=1 sortby id')
 
         """
+        if transform != None:
+            raise ValueError('transform is no longer supported: use json_depth=0 to disable JSON transformation')
+        schema_table = table.strip().split('.')
+        if len(schema_table) < 1 or len(schema_table) > 2:
+            raise ValueError('invalid table name: ' + table)
+        if json_depth is None or json_depth < 0 or json_depth > 4:
+            raise ValueError('invalid value for json_depth: ' + str(json_depth))
+        self._check_okapi()
+        self._check_db()
         if not self._quiet:
             print('ldlite: querying: '+path, file=sys.stderr)
         _autocommit(self.db, self.dbtype, False)
-        token = self._login()
-        cur = self.db.cursor()
-        cur.execute('DROP TABLE IF EXISTS '+table)
-        cur = self.db.cursor()
-        cur.execute('CREATE TABLE '+table+'(__id integer, jsonb varchar)')
-        hdr = { 'X-Okapi-Tenant': self.okapi_tenant,
-                'X-Okapi-Token': token }
+        curout = self.db.cursor()
+        if len(schema_table) == 2:
+            curout.execute('CREATE SCHEMA IF NOT EXISTS ' + _sqlid(schema_table[0]))
+        curout.execute('DROP TABLE IF EXISTS ' + _sqlid(table))
+        curout.execute('CREATE TABLE ' + _sqlid(table) + '(__id integer, jsonb varchar)')
         # First get total number of records
+        hdr = { 'X-Okapi-Tenant': self.okapi_tenant,
+                'X-Okapi-Token': self.login_token }
         resp = requests.get(self.okapi_url+path+'?offset=0&limit=1&query='+query, headers=hdr)
+        if resp.status_code == 401:
+            # Retry
+            self._login()
+            hdr = { 'X-Okapi-Tenant': self.okapi_tenant,
+                    'X-Okapi-Token': self.login_token }
+            resp = requests.get(self.okapi_url+path+'?offset=0&limit=1&query='+query, headers=hdr)
         if resp.status_code != 200:
             resp.raise_for_status()
         try:
@@ -189,7 +222,6 @@ class LDLite:
             else:
                 pbar = tqdm(desc='reading', total=total, leave=False, mininterval=1, smoothing=0, colour='#A9A9A9', bar_format='{desc} {bar}{postfix}')
             pbartotal = 0
-        cur = self.db.cursor()
         while True:
             offset = page * self.page_size
             limit = self.page_size
@@ -206,7 +238,7 @@ class LDLite:
             if lendata == 0:
                 break
             for d in data:
-                cur.execute('INSERT INTO '+table+' VALUES ('+str(count+1)+', \''+_escape_sql(json.dumps(d, indent=4))+'\')')
+                curout.execute('INSERT INTO ' + _sqlid(table) + ' VALUES(' + str(count+1) + ',\'' + _escape_sql(json.dumps(d, indent=4)) + '\')')
                 count += 1
                 if not self._quiet:
                     if pbartotal + 1 > total:
@@ -220,8 +252,9 @@ class LDLite:
         if not self._quiet:
             pbar.close()
         newtables = [table]
-        if transform:
-            newtables += _transform_json(self.db, table, count, self._quiet)
+        if json_depth > 0:
+            newtables += _transform_json(self.db, table, count, self._quiet, json_depth, curout)
+        self.db.commit()
         if not self._quiet:
             print('ldlite: created tables: '+', '.join(newtables), file=sys.stderr)
         _autocommit(self.db, self.dbtype, True)
@@ -258,6 +291,7 @@ class LDLite:
             ld.select(table='loans', columns=['id', 'item_id', 'loan_date'])
 
         """
+        self._check_db()
         _autocommit(self.db, self.dbtype, True)
         # f = sys.stdout if file is None else file
         f = sys.stdout
@@ -275,6 +309,7 @@ class LDLite:
             ld.to_csv(table='g', filename='g.csv')
 
         """
+        self._check_db()
         _autocommit(self.db, self.dbtype, True)
         _to_csv(self.db, table, filename)
 
